function A=RBF_QR_mat_1D(Psi,op,xe)
% Compute an RBF-QR matrix for the operator op, that is a_ij=L Psi_j(xe(i))
%--- op (string) : Can be '0', 'x', or 'xx' (which derivative)
%--- xe(1:Ne) : The evaluation points 
 
%--- Psi contains all the information about the RBF-QR basis    
  Ne = size(xe,1);
  ep = Psi.ep;
  Rt = Psi.Rt;
  jmax = Psi.jmax;
  M = Psi.M;
  N = size(Rt,1);
  
%--- Transform xe to 'polar' coordinates 
  re = abs(xe);
  se = ones(size(xe));
  pos = find(xe<0);
  se(pos) = -1;

%--- Evaluate the Chebyshev polynomials 
  Te = cos(acos(re(:,1))*(0:jmax));
  Te(:,1)=0.5*Te(:,1);
  
%--- These are only needed for the derivatives, but for clarity, compute here
  evec = exp(-ep^2*re.^2);
  evecx = 2*ep^2*re.*evec;
  evecx2 = 4*ep^4*re.^2.*evec;

  fac  = 1-re.^2;     
  pos = find(abs(re(:,1)-1)<eps); fac(pos)=1; % Problem spot(s)
  fac=1./fac;
   
  if (strcmp(op,'0'))
    Ve = (evec*ones(1,M)).*Te(:,1:M);
    % Multiply by the sign for all odd j
    for k=2:2:jmax+1
      Ve(:,k) = se.*Ve(:,k);
    end
    
  else
    %--- Compute the first derivative of the Chebyshev expansion fcns       
    dTe = -((re.*fac)*(0:jmax)).*Te;
    dTe(:,2) = 1; % Special case. Sabotaged by the scaling of T_0
    dTe(:,3:end) = dTe(:,3:end) + (fac*(2:jmax)).*Te(:,2:end-1);
    dTe(pos,:) = ones(length(pos),1)*(0:jmax).^2; % The problem spots
    
    if (strcmp(op,'x'))
     
    Ve = (evec*ones(1,M)).*dTe(:,1:M)-(evecx*ones(1,M)).*Te(:,1:M);

    for k=1:2:jmax+1
      % Inner derivative dr/dx=s^{-1} combined with s^p
      Ve(:,k) = se.*Ve(:,k);
    end
	
    elseif (strcmp(op,'xx'))
      %--- Compute the second derivative of the Chebyshev fcns    
      jj = (0:jmax);
      d2Te = -fac*jj.^2.*Te+(re(:,1).*fac)*ones(1,length(jj)).*dTe;
      d2Te(pos,:) = ones(length(pos),1)*(jj.^2.*(jj.^2-1)/3);
      d2Te(:,1:2) = 0; % Just making sure
      
      %--- Finally the second x-derivative
      Ve = (evec*ones(1,M)).*d2Te(:,1:M)- ...
           (2*evecx*ones(1,M)).*dTe(:,1:M)+ ...
           ((evecx2-2*ep^2*evec)*ones(1,M)).*Te(:,1:M);
      
      for k=2:2:jmax+1
        Ve(:,k) = se.*Ve(:,k);
      end	
    else
      error(['Operator ' op ' not implemented'])
    end    
  end

%--- Evaluate the RBF-QR matrix for the given operator 
  A = Ve(:,1:N) + Ve(:,N+1:M)*Rt.';


